<?
/*
Stronghold CMS - Database Schema Maintainer
Copyright (C) 2008 Scott Barnett

This program is free software; you can redistribute it and/or
modify it under the terms of Version 3 of the GNU General Public
License as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

----------------------------------------------------------------

$options=array(



	test
	drop_unknown_tables
	drop_unknown_constraints
)
*/

function install_schema($db,$txt_tables,$txt_constraints,$stored_procedures,$options){

	//echo ("<pre>");
	
	//pre($txt_tables);
	//pre($txt_constraints);
	
	$db_constraints=load_constraints($db);
	//pre($db_constraints);
	$db_tables=load_tables($db);
	//pre($db_tables);
	
	$sqls=array();
	// Scan for column problems
	
	// check for unknown tables
	foreach ($db_tables as $db_table_name => $db_table_attr){ //for each actual table
		if (!array_key_exists($db_table_name,$txt_tables)){
			echo ("ALERT: Unknown table ".$db_table_name."\n");
			if ($options["drop_unknown_tables"]){ $sqls[]="DROP TABLE ".$db_table_name; }
		}
	}
	
	// check for tables and problems with tables
	foreach ($txt_tables as $txt_table_name => $txt_table_attr){ //for each schema table
		$db_table_attr=$db_tables[$txt_table_name];
		//pre($db_table_attr);
		if (count($db_table_attr)>1){
	
			//check for missing attributes
			foreach ($txt_table_attr as $txt_attr_title => $txt_attr_properties){
				$db_attr_properties=$db_table_attr[$txt_attr_title];
	
				if (array_key_exists($txt_attr_title,$db_table_attr)){
					foreach ($txt_attr_properties as $txt_attr_property_title => $txt_attr_property_value){
						$db_attr_property_value=$db_attr_properties[$txt_attr_property_title];
					
						if (array_key_exists($txt_attr_property_title,$db_attr_properties)){
							if ($db_attr_properties[$txt_attr_property_title]==$txt_attr_property_value){
								/* perfect! */
							} else {
								if ($txt_attr_property_title=="is_nullable"){
									if ($txt_attr_property_value=="YES"){
										$sqls[]="ALTER TABLE ".$txt_table_name." ALTER COLUMN ".$txt_attr_title." DROP NOT NULL";
									} else {
										$sqls[]="ALTER TABLE ".$txt_table_name." ALTER COLUMN ".$txt_attr_title." SET NOT NULL";
									}
								} else {
									if ($txt_attr_property_value!="serial" && $db_attr_property_value!="integer"){ //ignore serial fields
										echo ("ALERT: Table [".$txt_table_name."] has attribute [".$txt_attr_title."] with property [".$txt_attr_property_title."=>".$db_attr_property_value."] which should be [".$txt_attr_property_title."=>".$txt_attr_property_value."]\n");
									}
								}
							}
						}
					}
				} else {
					// attribute is missing from the table
					$sqls[]="ALTER TABLE ".$txt_table_name." ADD ".$txt_attr_title." ".datatype($txt_attr_properties).";";
				}
			}
			
			//check for extra attributes
		} else {
			$sqls[]=sql_create_table($txt_table_name,$txt_table_attr);
		}
	}
	//pre($sqls);
	
	//-------------------------------------------------------------------------------------------------------------
	
	// check for unknown constraints
	foreach ($db_constraints as $db_constraint_name => $db_constraint_values){
		if (!array_key_exists($db_constraint_name,$txt_constraints)){
			echo ("ALERT: Unknown constraint ".$db_constraint_name."\n");
			if ($options["drop_unknown_constraints"]){ $sqls[]="ALTER TABLE ".$db_constraint_values["table_name"]." DROP CONSTRAINT ".$db_constraint_name; }
		}
	}
	
	// check for constraint problems and add constraints
	foreach ($txt_constraints as $txt_constraint_name => $txt_constraint_values){
		
		if (array_key_exists($txt_constraint_name,$db_constraints)){
			$db_constraint_values=$db_constraints[$txt_table_name];	
		} else {
			$sqls[]=sql_create_constraint($txt_constraint_name,$txt_constraint_values);
		}
		//foreach ($con as $id => $constraint){ $sqls[]="ALTER TABLE ".$prefix.$table_name." ADD CONSTRAINT ".$constraint.";"; }
	}
	
	//add the stored procedures
	foreach ($stored_procedures as $label => $sql){ $sqls[]=$sql; }	
	
	echo ("\n\n\n");
	//pre($sqls);
	foreach ($sqls as $id => $sql){
		
		if ($options["test"]){
			echo ("TEST SQL: ".$sql);	
		} else {
			echo ("EXECUTING SQL: ".$sql);
			$db->query($sql);
			echo (" DONE");
		}
		echo ("\n");
	}
	


}

//**************************************************************************************************************
// Utility functions

// populate the database with the initial data
function populate_database($db,$populate,$test=true){
	foreach ($populate as $table => $vals){
		foreach ($vals as $id => $insert){
			$sql="INSERT INTO ".$table." (".implode(",",array_keys($insert)).") VALUES (".implode(",",$insert).");";
			if ($test){
				echo ("TEST SQL: ".$sql."\n");
			} else {
				echo ("EXECUTING SQL: ".$sql."\n");
				$db->query($sql);
			}
		}
	}
}

//return the datatype SQL String
function datatype($properties){
	if (!is_array($properties)){ return (false); }
	
	if ($properties["data_type"]=="text"){
		$str="text";
		if ($properties["character_maximum_length"]){ $str.=" (".$properties["character_maximum_length"].")"; }
		return ($str);
	}

	if ($properties["data_type"]=="character varying"){
		$str="character varying";
		if ($properties["character_maximum_length"]){ $str.=" (".$properties["character_maximum_length"].")"; }
		return ($str);
	}
	
	if ($properties["data_type"]=="integer"){ return ("integer"); }
	
	if ($properties["data_type"]=="smallint"){ return ("smallint"); }
	
	//pre($properties);exit();

	return (NULL);
}


function load_constraints($db){

	$sql="SELECT c.constraint_type,u.table_name,u.column_name,u.constraint_name
	FROM information_schema.constraint_column_usage AS u, information_schema.table_constraints AS c
	WHERE u.table_schema='public'
	AND c.table_schema='public'
	AND u.constraint_name=c.constraint_name
	ORDER BY u.constraint_name ASC	";
	$rs=$db->query($sql);
	//echo ptable($rs);
	
	//create the constraints array
	$db_constraints=array();
	foreach ($rs as $id => $r){
		$constraint_name=$r["constraint_name"];
		if (!array_key_exists($constraint_name,$db_constraints)){
			$db_constraints[$constraint_name]=array("constraint_type"=>$r["constraint_type"],"table_name"=>$r["table_name"]);
		}
		$db_constraints[$constraint_name]["columns"][]=$r["column_name"];
	}

	return ($db_constraints);
}

function load_tables($db){
	// COLUMNS
	
	$sql="SELECT table_name,column_name,is_nullable,data_type,character_maximum_length FROM information_schema.columns WHERE table_schema='public'";
	$rs=$db->query($sql);
	
	//create the schema array
	$db_tables=array();
	foreach($rs as $id => $r){
		$table_name=$r["table_name"];
		$column_name=$r["column_name"];
		unset($r["table_name"]);
		unset($r["column_name"]);
		$db_tables[$table_name][$column_name]=$r;
	}
	return ($db_tables);
}

//pre($schema);

//return the SQL required to add a constraint
function sql_create_constraint($name,$values){
	$sql=NULL;
	if ($values["constraint_type"]=="PRIMARY"){ $sql="ALTER TABLE ".$values["table_name"]." ADD PRIMARY KEY (".implode(",",$values["columns"]).")";	}
	elseif ($values["constraint_type"]=="UNIQUE"){ $sql="ALTER TABLE ".$values["table_name"]." ADD CONSTRAINT ".$name." UNIQUE (".implode(",",$values["columns"]).")"; }
	else { echo  ("ALERT: Unkown constraint type -> ".$values["constraint_type"]); }
	return ($sql);
}

function sql_create_table($name,$attr){
	$sql="CREATE TABLE ".$name." (";
	
	$bits=array();
	foreach ($attr as $attr_name => $properties ){
		$bit=$attr_name." ".$properties["data_type"];
		if ($properties["character_maximum_length"]){ $bit.="(".$properties["character_maximum_length"].")"; }
		if ($properties["is_nullable"]=="NO"){ $bit.=" NOT NULL"; }
		$bits[]=$bit;
	}
	$sql.=implode(",",$bits).")";

	return ($sql);
}

?>
