<?
/*
Stronghold CMS - Stronghold Database Schema
Copyright (C) 2008 Scott Barnett

This program is free software; you can redistribute it and/or
modify it under the terms of Version 3 of the GNU General Public
License as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.


------------------------------------------------------------------

At this stage, the schema-matic only works with Postgres.

Defining the schema in this way is intended to provide the basis for
supporting multiple RDBMS'

This script can be included multiple times

INPUTS
$db_common_prefix
$prefixes = an array of db_table_prefixes
$populate_database
*/

//check the inputs were defined
if (!$db_common_prefix){ echo ("FATAL: No common specified\n"); exit(); }
if (!is_array($prefixes)){ echo ("FATAL: No prefixes specified (must be an array)\n"); exit(); }
if ($populate_database===NULL){ echo ("FATAL: Population flag not set\n"); exit(); }
if ($database_type==NULL){ echo ("FATAL: Database type flag not set\n"); exit(); }

//reset the variables
$txt_tables=array();
$txt_constraints=array();
$populate=array();

//******************************************************************************************************************************************
// Common Tables


$table_name="users";
$txt_tables[$db_common_prefix.$table_name]=array(
	"user_id"		=>array("is_nullable" => "NO", 	"data_type" => "serial", 			"character_maximum_length" => NULL),
	"username"		=>array("is_nullable" => "NO", 	"data_type" => "character varying",	"character_maximum_length" => 100),
	"password"		=>array("is_nullable" => "YES",	"data_type" => "character varying",	"character_maximum_length" => 50),
	"firstname"		=>array("is_nullable" => "NO", 	"data_type" => "character varying",	"character_maximum_length" => 255),
	"surname"		=>array("is_nullable" => "NO", 	"data_type" => "character varying", "character_maximum_length" => 255),
	"email"			=>array("is_nullable" => "NO", 	"data_type" => "character varying",	"character_maximum_length" => 255),
	"access"		=>array("is_nullable" => "NO", 	"data_type" => "smallint", 			"character_maximum_length" => NULL),
	"active"		=>array("is_nullable" => "NO", 	"data_type" => "smallint", 			"character_maximum_length" => NULL),
	"editor_size"	=>array("is_nullable" => "NO", 	"data_type" => "character varying", "character_maximum_length" => 11),
);
$txt_constraints[$db_common_prefix.$table_name."_pkey"]=array("constraint_type"=>"PRIMARY","table_name"=>$db_common_prefix.$table_name,"columns"=>array("user_id"));
$txt_constraints[$db_common_prefix.$table_name."_unique"]=array("constraint_type"=>"UNIQUE","table_name"=>$db_common_prefix.$table_name,"columns"=>array("username"));

//-------------------------------------------------------------------------------------------------------------------------

$table_name="refs";
$txt_tables[$db_common_prefix.$table_name]=array(
	"ref"			=>array("is_nullable" => "NO", 	"data_type" => "character varying",	"character_maximum_length" => 10),
	"ref_id"		=>array("is_nullable" => "NO",	"data_type" => "integer",			"character_maximum_length" => NULL),
	"ref_val"		=>array("is_nullable" => "NO",	"data_type" => "character varying",	"character_maximum_length" => 20),
);
$txt_constraints[$db_common_prefix.$table_name."_unique"]=array("constraint_type"=>"UNIQUE","table_name"=>$db_common_prefix.$table_name,"columns"=>array("ref","ref_id"));

//-------------------------------------------------------------------------------------------------------------------------

$table_name="seq";
$txt_tables[$db_common_prefix.$table_name]=array(
	"seq_name"		=>array("is_nullable" => "NO",	"data_type" => "character varying",	"character_maximum_length" => 10),
	"seq_val"		=>array("is_nullable" => "NO",	"data_type" => "integer",			"character_maximum_length" => NULL),
);
$txt_constraints[$db_common_prefix.$table_name."_unique"]=array("constraint_type"=>"UNIQUE","table_name"=>$db_common_prefix.$table_name,"columns"=>array("seq_name"));

//-------------------------------------------------------------------------------------------------------------------------

$table_name="tbl";
$txt_tables[$db_common_prefix.$table_name]=array(
	"tbl_id"		=>array("is_nullable" => "NO", 	"data_type" => "serial",			"character_maximum_length" => NULL),
	"major"			=>array("is_nullable" => "NO",	"data_type" => "smallint",			"character_maximum_length" => NULL),
	"minor"			=>array("is_nullable" => "NO",	"data_type" => "smallint",			"character_maximum_length" => NULL),
	"def"			=>array("is_nullable" => "NO",	"data_type" => "character varying",	"character_maximum_length" => 100),
);
$txt_constraints[$db_common_prefix.$table_name."_pkey"]=array("constraint_type"=>"PRIMARY","table_name"=>$db_common_prefix.$table_name,"columns"=>array("tbl_id"));

//******************************************************************************************************************************************
// Tables for each CMS

foreach ($prefixes as $id => $prefix){
	//-------------------------------------------------------------------------------------------------------------------------

	$table_name="acl";
	$txt_tables[$prefix.$table_name]=array(
		"group_id"		=>array("is_nullable" => "NO", 	"data_type" => "integer", 			"character_maximum_length" => NULL),
		"page_id"		=>array("is_nullable" => "NO", 	"data_type" => "integer", 			"character_maximum_length" => NULL),
		"access"		=>array("is_nullable" => "NO", 	"data_type" => "integer", 			"character_maximum_length" => NULL),
	);
	$txt_constraints[$prefix.$table_name."_unique"]=array("constraint_type"=>"UNIQUE","table_name"=>$prefix.$table_name,"columns"=>array("group_id","page_id"));
	
	//-------------------------------------------------------------------------------------------------------------------------
	
	$table_name="approvals";
	$txt_tables[$prefix.$table_name]=array(
		"page_id"		=>array("is_nullable" => "NO", 	"data_type" => "integer", 			"character_maximum_length" => NULL),
		"version_id"	=>array("is_nullable" => "NO", 	"data_type" => "integer", 			"character_maximum_length" => NULL),
		"group_id"		=>array("is_nullable" => "NO", 	"data_type" => "integer", 			"character_maximum_length" => NULL),
		"timestamp"		=>array("is_nullable" => "NO", 	"data_type" => "integer", 			"character_maximum_length" => NULL),
		"user_id"		=>array("is_nullable" => "NO", 	"data_type" => "integer", 			"character_maximum_length" => NULL),
	);
	$txt_constraints[$prefix.$table_name."_unique"]=array("constraint_type"=>"UNIQUE","table_name"=>$prefix.$table_name,"columns"=>array("page_id","version_id","group_id"));
	
	//-------------------------------------------------------------------------------------------------------------------------
	
	$table_name="editlog";
	$txt_tables[$prefix.$table_name]=array(
		"page_id"		=>array("is_nullable" => "NO", 	"data_type" => "integer", 			"character_maximum_length" => NULL),
		"version_id"	=>array("is_nullable" => "NO",	"data_type" => "integer",			"character_maximum_length" => NULL),
		"user_id"		=>array("is_nullable" => "NO",	"data_type" => "integer",			"character_maximum_length" => NULL),
		"timestamp"		=>array("is_nullable" => "NO",	"data_type" => "integer",			"character_maximum_length" => NULL),
		"event"			=>array("is_nullable" => "NO",	"data_type" => "smallint",			"character_maximum_length" => NULL),
	);
	
	//-------------------------------------------------------------------------------------------------------------------------
	
	$table_name="group_members";
	$txt_tables[$prefix.$table_name]=array(
		"group_id"		=>array("is_nullable" => "NO", 	"data_type" => "integer", 			"character_maximum_length" => NULL),
		"user_id"		=>array("is_nullable" => "NO", 	"data_type" => "integer", 			"character_maximum_length" => NULL),
		"level"			=>array("is_nullable" => "NO", 	"data_type" => "smallint", 			"character_maximum_length" => NULL),
	);
	$txt_constraints[$prefix.$table_name."_unique"]=array("constraint_type"=>"UNIQUE","table_name"=>$prefix.$table_name,"columns"=>array("group_id","user_id"));
	
	//-------------------------------------------------------------------------------------------------------------------------
	
	$table_name="groups";
	$txt_tables[$prefix.$table_name]=array(
		"group_id"		=>array("is_nullable" => "NO", 	"data_type" => "serial", 			"character_maximum_length" => NULL),
		"title"			=>array("is_nullable" => "NO", 	"data_type" => "character varying", "character_maximum_length" => 255),
	);
	$txt_constraints[$prefix.$table_name."_pkey"]=array("constraint_type"=>"PRIMARY","table_name"=>$prefix.$table_name,"columns"=>array("group_id"));

	//-------------------------------------------------------------------------------------------------------------------------
	
	$table_name="locks";
	$txt_tables[$prefix.$table_name]=array(
		"page_id"		=>array("is_nullable" => "NO", 	"data_type" => "integer", 			"character_maximum_length" => NULL),
		"user_id"		=>array("is_nullable" => "NO",	"data_type" => "integer",			"character_maximum_length" => NULL),
		"version_id"	=>array("is_nullable" => "NO",	"data_type" => "integer",			"character_maximum_length" => NULL),
		"timestamp"		=>array("is_nullable" => "NO",	"data_type" => "integer",			"character_maximum_length" => NULL),
	);
	$txt_constraints[$prefix.$table_name."_unique"]=array("constraint_type"=>"UNIQUE","table_name"=>$prefix.$table_name,"columns"=>array("page_id","user_id","version_id"));

	//-------------------------------------------------------------------------------------------------------------------------
	
	$table_name="module";
	$txt_tables[$prefix.$table_name]=array(
		"page_id"		=>array("is_nullable" => "NO", 	"data_type" => "integer",			"character_maximum_length" => NULL),
		"version_id"	=>array("is_nullable" => "NO", 	"data_type" => "integer",			"character_maximum_length" => NULL),
		"title"			=>array("is_nullable" => "YES",	"data_type" => "character varying",	"character_maximum_length" => 255),
		"mod_time"		=>array("is_nullable" => "NO", 	"data_type" => "integer",			"character_maximum_length" => NULL),
		"mod_userid"	=>array("is_nullable" => "YES",	"data_type" => "integer",			"character_maximum_length" => NULL),
		"status"		=>array("is_nullable" => "NO",	"data_type" => "smallint",			"character_maximum_length" => NULL),
		"created_date"	=>array("is_nullable" => "YES", "data_type" => "integer",			"character_maximum_length" => NULL),
		"short_title"	=>array("is_nullable" => "YES", "data_type" => "character varying",	"character_maximum_length" => 20),
		"nav_id"		=>array("is_nullable" => "NO",  "data_type" => "smallint",			"character_maximum_length" => NULL),
		"heading"		=>array("is_nullable" => "YES", "data_type" => "smallint",			"character_maximum_length" => NULL),
		"publish_live"	=>array("is_nullable" => "YES", "data_type" => "integer",			"character_maximum_length" => NULL),
		"publish_end"	=>array("is_nullable" => "YES", "data_type" => "integer",			"character_maximum_length" => NULL),
	);
	$txt_constraints[$prefix.$table_name."_unique"]=array("constraint_type"=>"UNIQUE","table_name"=>$prefix.$table_name,"columns"=>array("page_id","version_id"));
	
	//-------------------------------------------------------------------------------------------------------------------------
	
	$table_name="module_plugin";
	$txt_tables[$prefix.$table_name]=array(
		"page_id"		=>array("is_nullable" => "NO", 	"data_type" => "integer",			"character_maximum_length" => NULL),
		"version_id"	=>array("is_nullable" => "NO", 	"data_type" => "integer",			"character_maximum_length" => NULL),
		"include_path"	=>array("is_nullable" => "YES",	"data_type" => "text",				"character_maximum_length" => NULL),
		"notes"			=>array("is_nullable" => "YES", "data_type" => "text",				"character_maximum_length" => NULL),
	);
	$txt_constraints[$prefix.$table_name."_unique"]=array("constraint_type"=>"UNIQUE","table_name"=>$prefix.$table_name,"columns"=>array("page_id","version_id"));
	
	//-------------------------------------------------------------------------------------------------------------------------
	
	$table_name="module_sitemap";
	$txt_tables[$prefix.$table_name]=array(
		"page_id"		=>array("is_nullable" => "NO", 	"data_type" => "integer",			"character_maximum_length" => NULL),
		"version_id"	=>array("is_nullable" => "NO", 	"data_type" => "integer",			"character_maximum_length" => NULL),
		"text_before"	=>array("is_nullable" => "YES",	"data_type" => "text",				"character_maximum_length" => NULL),
		"text_after"	=>array("is_nullable" => "YES",	"data_type" => "text",				"character_maximum_length" => NULL),
	);
	$txt_constraints[$prefix.$table_name."_unique"]=array("constraint_type"=>"UNIQUE","table_name"=>$prefix.$table_name,"columns"=>array("page_id","version_id"));
	
	//-------------------------------------------------------------------------------------------------------------------------
	
	$table_name="module_text";
	$txt_tables[$prefix.$table_name]=array(
		"page_id"		=>array("is_nullable" => "NO", 	"data_type" => "integer",			"character_maximum_length" => NULL),
		"version_id"	=>array("is_nullable" => "NO", 	"data_type" => "integer",			"character_maximum_length" => NULL),
		"body_text"		=>array("is_nullable" => "YES",	"data_type" => "text",				"character_maximum_length" => NULL),
		"notes"			=>array("is_nullable" => "YES", "data_type" => "text",				"character_maximum_length" => NULL),
	);
	$txt_constraints[$prefix.$table_name."_unique"]=array("constraint_type"=>"UNIQUE","table_name"=>$prefix.$table_name,"columns"=>array("page_id","version_id"));
	
	//-------------------------------------------------------------------------------------------------------------------------

	$table_name="modules";
	$txt_tables[$prefix.$table_name]=array(
		"module_id"		=>array("is_nullable" => "NO", 	"data_type" => "integer", 			"character_maximum_length" => NULL),
		"module_name"	=>array("is_nullable" => "NO",	"data_type" => "character varying",	"character_maximum_length" => 20),
		"module_desc"	=>array("is_nullable" => "NO",	"data_type" => "character varying",	"character_maximum_length" => 255),
		"time_installed"=>array("is_nullable" => "NO",	"data_type" => "integer",			"character_maximum_length" => NULL),
	);
	$txt_constraints[$prefix.$table_name."_pkey"]=array("constraint_type"=>"PRIMARY","table_name"=>$prefix.$table_name,"columns"=>array("module_id"));

	//-------------------------------------------------------------------------------------------------------------------------
	
	$table_name="pages";
	$txt_tables[$prefix.$table_name]=array(
		"page_id"		=>array("is_nullable" => "NO", 	"data_type" => "serial", 			"character_maximum_length" => NULL),
		"time_created"	=>array("is_nullable" => "NO", 	"data_type" => "integer", 			"character_maximum_length" => NULL),
		"parent"		=>array("is_nullable" => "NO",	"data_type" => "integer", 			"character_maximum_length" => NULL),
		"workflow_id"	=>array("is_nullable" => "NO", 	"data_type" => "integer", 			"character_maximum_length" => NULL),
		"link"			=>array("is_nullable" => "YES",	"data_type" => "character varying",	"character_maximum_length" => 255),
		"url"			=>array("is_nullable" => "NO", 	"data_type" => "character varying",	"character_maximum_length" => 255),
		"module_id"		=>array("is_nullable" => "NO", 	"data_type" => "integer", 			"character_maximum_length" => NULL),
		"archive_date"	=>array("is_nullable" => "YES",	"data_type" => "integer", 			"character_maximum_length" => NULL),
		"levels"		=>array("is_nullable" => "NO", 	"data_type" => "smallint",			"character_maximum_length" => NULL),
	);
	$txt_constraints[$prefix.$table_name."_pkey"]=array("constraint_type"=>"PRIMARY","table_name"=>$prefix.$table_name,"columns"=>array("page_id"));
	
	//-------------------------------------------------------------------------------------------------------------------------
	
	$table_name="resources";
	$txt_tables[$prefix.$table_name]=array(
		"resource_id"		=>array("is_nullable" => "NO", 	"data_type" => "serial", 			"character_maximum_length" => NULL),
		"current_version"	=>array("is_nullable" => "NO", 	"data_type" => "integer", 			"character_maximum_length" => NULL),
		"time_created"		=>array("is_nullable" => "NO",	"data_type" => "integer", 			"character_maximum_length" => NULL),
		"workflow_id"		=>array("is_nullable" => "NO",	"data_type" => "integer", 			"character_maximum_length" => NULL),
		"archive_date"		=>array("is_nullable" => "YES",	"data_type" => "integer", 			"character_maximum_length" => NULL),
	);
	$txt_constraints[$prefix.$table_name."_pkey"]=array("constraint_type"=>"PRIMARY","table_name"=>$prefix.$table_name,"columns"=>array("resource_id"));

	//-------------------------------------------------------------------------------------------------------------------------
	
	$table_name="resource_approvals";
	$txt_tables[$prefix.$table_name]=array(
		"resource_id"	=>array("is_nullable" => "NO", 	"data_type" => "integer", 			"character_maximum_length" => NULL),
		"version_id"	=>array("is_nullable" => "NO", 	"data_type" => "integer", 			"character_maximum_length" => NULL),
		"group_id"		=>array("is_nullable" => "NO", 	"data_type" => "integer", 			"character_maximum_length" => NULL),
		"timestamp"		=>array("is_nullable" => "NO", 	"data_type" => "integer", 			"character_maximum_length" => NULL),
		"user_id"		=>array("is_nullable" => "NO", 	"data_type" => "integer", 			"character_maximum_length" => NULL),
	);
	$txt_constraints[$prefix.$table_name."_unique"]=array("constraint_type"=>"UNIQUE","table_name"=>$prefix.$table_name,"columns"=>array("resource_id","version_id","group_id"));

	//-------------------------------------------------------------------------------------------------------------------------

	$table_name="resource_log";
	$txt_tables[$prefix.$table_name]=array(
		"resource_id"	=>array("is_nullable" => "NO", 	"data_type" => "integer", 			"character_maximum_length" => NULL),
		"version_id"	=>array("is_nullable" => "NO",	"data_type" => "integer",			"character_maximum_length" => NULL),
		"user_id"		=>array("is_nullable" => "NO",	"data_type" => "integer",			"character_maximum_length" => NULL),
		"timestamp"		=>array("is_nullable" => "NO",	"data_type" => "integer",			"character_maximum_length" => NULL),
		"event"			=>array("is_nullable" => "NO",	"data_type" => "smallint",			"character_maximum_length" => NULL),
	);
	
	//-------------------------------------------------------------------------------------------------------------------------
	
	$table_name="resource_versions";
	$txt_tables[$prefix.$table_name]=array(
		"resource_id"		=>array("is_nullable" => "NO", 	"data_type" => "integer", 			"character_maximum_length" => NULL),
		"version_id"		=>array("is_nullable" => "NO", 	"data_type" => "integer", 			"character_maximum_length" => NULL),
		"node_id"			=>array("is_nullable" => "NO", 	"data_type" => "integer", 			"character_maximum_length" => NULL),
		"filename"			=>array("is_nullable" => "NO", 	"data_type" => "character varying",	"character_maximum_length" => 100),
		"resource_name"		=>array("is_nullable" => "YES",	"data_type" => "character varying",	"character_maximum_length" => 50),
		"file_type"			=>array("is_nullable" => "NO",	"data_type" => "character",			"character_maximum_length" => 3),
		"time_created"		=>array("is_nullable" => "NO",	"data_type" => "integer", 			"character_maximum_length" => NULL),
		"status"			=>array("is_nullable" => "NO",	"data_type" => "smallint", 			"character_maximum_length" => NULL),
		"md5_sum"			=>array("is_nullable" => "NO",	"data_type" => "character", 		"character_maximum_length" => 32),
		"notes"				=>array("is_nullable" => "NO",	"data_type" => "text",				"character_maximum_length" => NULL),
	);
	$txt_constraints[$prefix.$table_name."_unique"]=array("constraint_type"=>"UNIQUE","table_name"=>$prefix.$table_name,"columns"=>array("resource_id","version_id"));
//	$txt_constraints[$prefix.$table_name."_uniquename"]=array("constraint_type"=>"UNIQUE","table_name"=>$prefix.$table_name,"columns"=>array("resource_id","version_id"));
	
	//-------------------------------------------------------------------------------------------------------------------------

	$table_name="resource_tree";
	$txt_tables[$prefix.$table_name]=array(
		"node_id"		=>array("is_nullable" => "NO", 	"data_type" => "serial", 			"character_maximum_length" => NULL),
		"node_name"		=>array("is_nullable" => "NO", 	"data_type" => "character varying",	"character_maximum_length" => 50),
		"node_parent"	=>array("is_nullable" => "NO",	"data_type" => "integer", 			"character_maximum_length" => NULL),
	);
	$txt_constraints[$prefix.$table_name."_pkey"]=array("constraint_type"=>"PRIMARY","table_name"=>$prefix.$table_name,"columns"=>array("node_id"));
	
	//-------------------------------------------------------------------------------------------------------------------------
	
	$table_name="workflow";
	$txt_tables[$prefix.$table_name]=array(
		"workflow_id"	=>array("is_nullable" => "NO", 	"data_type" => "integer", 			"character_maximum_length" => NULL),
		"group_id"		=>array("is_nullable" => "NO", 	"data_type" => "integer",			"character_maximum_length" => NULL),
	);
	$txt_constraints[$prefix.$table_name."_unique"]=array("constraint_type"=>"UNIQUE","table_name"=>$prefix.$table_name,"columns"=>array("workflow_id","group_id"));
	
	//-------------------------------------------------------------------------------------------------------------------------
}

//********************************************************************************************************************************
// Initial data population

if ($populate_database){
	$now=time();

	foreach ($prefixes as $id => $prefix){
		$populate[$prefix."pages"][]=array(	
		//	page_id is a sequence, presume it's at 0
			"time_created"=>$now,
			"parent"=>"0",
			"workflow_id"=>1,
			"link"=>"NULL",
			"url"=>"'/'",
			"module_id"=>1,
			"levels"=>0,
		);
		
		$populate[$prefix."module"][]=array(
			"page_id"=>1,
			"version_id"=>1,
			"title"=>"'Home Page'",
			"mod_time"=>$now,
			"mod_userid"=>1,
			"status"=>1,
			"created_date"=>$now,
			"short_title"=>"'Home'",
			"nav_id"=>1,
			"heading"=>1,
		);
		
		//default document text!
$default_doc="'
<p>Thankyou for installing Stronghold CMS! Some things you may like to know:</p>

<ul>
<li><strong>Website Skin</strong><br>
Stronghold CMS is really designed to be as visually flexible as possible. As such, the default website
skin for a new CMS is very basic, it&#39;s designed to be a blank slate for you to get creative with. :)
In the &quot;includes&quot; folder are two files, template.php and style.css. They provide some basic and
easy to understand examples for you to work from.
<br><br></li>

<li><strong>Default Workflow</strong><br>
The default workflow has been established as just your group. You should give some thought to which groups in your
organisation should be needed to approve content. In most organisations, it&#39;s both Public Relations and the IT department.
<br><br></li>

<li><strong>Backups</strong><br>
The installer has not established a backup routine for you. You should look into this as part of your organisational backup
scheme. You can find information that may assist in the
<a href=\"http://www.strongholdcms.org/wiki/doku.php?id=administration:backups\" target=\"_blank\">Administration section of the Stronghold CMS Manual</a>.
<br><br></li>

<li><strong>Database Access</strong><br>
If you&#39;ve used the automated installation, PostgreSQL is accessible via <a href=\"/phpPgAdmin/\" target=\"_blank\">phpPgAdmin</a>.<br>
Username: postgres<br>
Password: (your database password)
<br><br></li>

<li><strong>Colouring Outside the Square</strong><br>
<a href=\"http://www.strongholdcms.org/wiki/\" target=\"_blank\">The Stronghold CMS Manual</a> is a great place to look for
formal information and structured information, you may even be able to help us improve it :)<br>
<a href=\"http://www.strongholdcms.org/wiki/doku.php?id=administration:backups\" target=\"_blank\">The Stronghold CMS Forums</a>
are a great place to discuss everything Content Management with system owners, developers, and graphic artists.
<br><br></li>
</ul>


'";
		
		$populate[$prefix."module_text"][]=array("page_id"=>1,"version_id"=>1,"body_text"=>$default_doc);
		
		$populate[$prefix."modules"][]=array("module_id"=>1,"module_name"=>"'text'","module_desc"=>"'Generic Text Module'","time_installed"=>$now);
		
		$populate[$prefix."editlog"][]=array("page_id"=>1,"version_id"=>1,"user_id"=>1,"timestamp"=>$now,"event"=>7); //technically this should be done *after* the installer?
	
		//$populate[$prefix."groups"][]=array("group_id"=>1,"title"=>"'".$CONFIG["system_tech"]."'",); //group primary key increments automatically

		//$populate[$prefix."workflow"][]=array("workflow_id"=>1,"group_id"=>1);
		
		//$populate[$prefix."group_members"][]=array("group_id"=>1,"user_id"=>1,"level"=>3);
		
		$populate[$prefix."resource_tree"][]=array("node_id"=>1,"node_name"=>"'resources'","node_parent"=>0);
		$populate[$prefix."resource_tree"][]=array("node_id"=>2,"node_name"=>"'docs'","node_parent"=>1);
		$populate[$prefix."resource_tree"][]=array("node_id"=>3,"node_name"=>"'images'","node_parent"=>1);
	}
}

if ($populate_common){
	
	//editlog definitions
	$populate[$db_common_prefix."refs"][]=array("ref"=>"'editlog'","ref_id"=>"1","ref_val"=>"'UPDATE'");
	$populate[$db_common_prefix."refs"][]=array("ref"=>"'editlog'","ref_id"=>"2","ref_val"=>"'ARCHIVE'");
	$populate[$db_common_prefix."refs"][]=array("ref"=>"'editlog'","ref_id"=>"3","ref_val"=>"'APPROVE'");
	$populate[$db_common_prefix."refs"][]=array("ref"=>"'editlog'","ref_id"=>"4","ref_val"=>"'LIVE'");
	$populate[$db_common_prefix."refs"][]=array("ref"=>"'editlog'","ref_id"=>"5","ref_val"=>"'REV_NOTE'");
	$populate[$db_common_prefix."refs"][]=array("ref"=>"'editlog'","ref_id"=>"6","ref_val"=>"'SUBMIT'");
	$populate[$db_common_prefix."refs"][]=array("ref"=>"'editlog'","ref_id"=>"7","ref_val"=>"'CREATE'");
	$populate[$db_common_prefix."refs"][]=array("ref"=>"'editlog'","ref_id"=>"8","ref_val"=>"'CANCEL'");
	
	//user access definitions
	$populate[$db_common_prefix."refs"][]=array("ref"=>"'useraccess'","ref_id"=>"1","ref_val"=>"'Editor'");
	$populate[$db_common_prefix."refs"][]=array("ref"=>"'useraccess'","ref_id"=>"2","ref_val"=>"'Approver'");
	$populate[$db_common_prefix."refs"][]=array("ref"=>"'useraccess'","ref_id"=>"3","ref_val"=>"'Custodian'");
	$populate[$db_common_prefix."refs"][]=array("ref"=>"'useraccess'","ref_id"=>"4","ref_val"=>"'Administrator'");

	//status definitions
	$populate[$db_common_prefix."refs"][]=array("ref"=>"'status'","ref_id"=>"1","ref_val"=>"'New'");
	$populate[$db_common_prefix."refs"][]=array("ref"=>"'status'","ref_id"=>"2","ref_val"=>"'Pending'");
	$populate[$db_common_prefix."refs"][]=array("ref"=>"'status'","ref_id"=>"3","ref_val"=>"'Approved'");
	$populate[$db_common_prefix."refs"][]=array("ref"=>"'status'","ref_id"=>"4","ref_val"=>"'Cancelled'");

	//sequences (emulated via stored procedure)
	$populate[$db_common_prefix."seq"][]=array("seq_name"=>"'users'","seq_val"=>"1");
	$populate[$db_common_prefix."seq"][]=array("seq_name"=>"'groups'","seq_val"=>"1");
	$populate[$db_common_prefix."seq"][]=array("seq_name"=>"'pages'","seq_val"=>"1");
	$populate[$db_common_prefix."seq"][]=array("seq_name"=>"'nodes'","seq_val"=>"1");
	$populate[$db_common_prefix."seq"][]=array("seq_name"=>"'resources'","seq_val"=>"1");

}

//****************************************************************************************************************
/*
I'm pretty resigned to the fact that stored procedures aren't going to be consistent across different
database technologies. If you can contribute to other technologies in this area, please email your
procedures to scott@strongholdcms.org. Thanks.
*/

$stored_procedures=array();
if ($database_type=="postgres"){

$stored_procedures["tree_path"]="
CREATE FUNCTION tree_path(integer, character varying) RETURNS character varying
    AS 'DECLARE
top_node_id ALIAS FOR $1;
db_table ALIAS FOR $2;
path varchar(500);
node int;
i int;
i_node_name varchar(255);
i_node_parent int;
itemrecord RECORD;

BEGIN

   i:=1;
   node:=top_node_id;
   path:='''';


   WHILE i<100 LOOP

      i_node_name:=NULL;
      i_node_parent:=NULL;

      FOR itemrecord IN EXECUTE (''SELECT node_name,node_parent FROM ''|| db_table ||'' AS t WHERE node_id=''||node) LOOP
         i_node_name:=itemrecord.node_name;
         i_node_parent:=itemrecord.node_parent;
      END LOOP;

      IF i_node_name IS NULL THEN
         RETURN (''busted'');
      END IF;

      path:=''/'' || i_node_name || path;

      IF i_node_parent=0 THEN
         RETURN (path);
      END IF;

      node := i_node_parent;

      i := i+1;
   END LOOP;

   RETURN path;

END '
    LANGUAGE plpgsql;
";
}
?>
